# Simple UPnP control point using BRisa framework

from brisa.core.reactors import install_default_reactor
reactor = install_default_reactor()

from brisa.core.network import parse_url
from brisa.core.threaded_call import run_async_function

from brisa.upnp.control_point.control_point import ControlPoint
from brisa.upnp.control_point.control_point_av import ControlPointAV

service = ('u','urn:schemas-upnp-org:service:PlayList:1')
zukebox_type = 'urn:schemas-upnp-org:device:ZukeBoxServer:1'

class ZukeboxControlPoint(ControlPointAV):

    devices=[]

    def __init__(self):
	ControlPointAV.__init__(self)
	devices=[]

    def start(self):

        self.cp = self.create_control_point()
        self.cp.start()
	print 'Create control point'
        #reactor.add_after_stop_func(self.cp.stop)
        #reactor.main()
	#print 'Reactor main() criado'
	self.cp.start_search(300, 'upnp:rootdevice')
	print 'Start_search'
        while True:
	    if self.set_zukebox_server(self.cp):
	        print 'Searching Zukebox Server'
		break

	print 'After searching'
 	print self.cp.get_current_server().services()
	return self.cp


    def create_control_point(self):
        """ Creates the control point and binds callbacks to device events.
        """
        cp = ControlPointAV()
        cp.subscribe('new_device_event', self.on_new_device)
        cp.subscribe('removed_device_event', self.on_removed_device)
        return cp


    def on_new_device(self, dev):

        if not dev:
            return

        devices.append(dev)


    def on_removed_device(self, udn):

        for dev in devices:
	    if dev.udn == udn:
	        devices.remove(dev)


    def get_switch_service(self, device):
        return device.services[service[1]]


    def set_zukebox_server(self, cp):
        
        for device in cp.get_devices():
	    print 'set_zukebox_server'
            if device.values().friendly_name == 'Zukebox-Server':
                cp.set_current_server(device)
                return True

        return False


    def get_playlist(self, cp):

	playlist = {}
	try:
            device = cp.get_current_server()
            service = device.get_service_by_type('urn:schemas-upnp-org:service:Playlist:1')
            playlist = service.GetPlaylist()
	except Exception, e:
            if not service:
                print 'Service not discovered. Exception: %s' %e
            elif not playlist:
	        print 'Error in get current playlist. Exception: %s' %e
	    else:
		print '%s'%e
	    playlist = None
	
	return playlist


    def searchGenreList(self, cp, genreType):

	genre = {}
	try:
	    device = cp.get_current_server()
	    service = device.get_service_by_type('urn:schemas-upnp-org:service:Search:1')
	    GenreType = {'GenreType':genreType}
	    genre = service.SearchGenreList(GenreType)
	except Exception, e:
	    if not service:
		print 'Service not discovered. Exception: %s' %e
	    elif not genre:
		print 'Error in get genre type list. Exception: %s' %e
	    else:
	        print '%s' %e
	    genre = None

	return genre


    def searchMusicList(self, cp, music_name):

	music = {}
	try:
	    device = cp.get_current_server()
	    service = device.get_service_by_type('urn:schemas-upnp-org:service:Search:1')
	    MusicName = {'MusicName':music_name}
	    music = service.SearchMusicList(MusicName)
	except Exception, e:
	    if not service:
		print 'Service not discovered. Exception: %s' %e
	    elif not music:
		print 'Error in get music list by name. Exception: %s' %e
	    else:
	        print '%s' %e
	    music = None

	return music
	    

    def searchArtistList(self, cp, artist_name):

	artist = {}
	try:
	    device = cp.get_current_server()
	    service = device.get_service_by_type('urn:schemas-upnp-org:service:Search:1')
	    ArtistName = {'ArtistName':artist_name}
	    artist = service.SearchMusicList(ArtistName)
	except Exception, e:
	    if not service:
		print 'Service not discovered. Exception: %s' %e
	    elif not artist:
		print 'Error in get artist list by name. Exception: %s' %e
	    else:
	        print '%s' %e
	    artist = None

	return artist


    def searchAllAudioList(self, cp):

	all_audio = {}
	try:
	    device = cp.get_current_server()
	    service = device.get_service_by_type('urn:schemas-upnp-org:service:Search:1')
	    all_audio = service.SearchMusicList()
	except Exception, e:
	    if not service:
		print 'Service not discovered. Exception: %s' %e
	    elif not all_audio:
		print 'Error in get music list by name. Exception: %s' %e
	    else:
	        print '%s' %e
	    all_audio = None

	return all_audio


    def content_directory(self, cp):
        print 'Testing content_directory acess '


    def stop(self, cp):
        cp.stop()
        reactor.main_quit()
