
import os
#from brisa.core.reactors import install_default_reactor
#reactor = install_default_reactor()

from brisa.core import log
from brisa.upnp.device import Service, ServiceController

pjoin = os.path.join

class PlayListOutBoundExcept(Exception):
    def __init__(self, *args, **kwargs):
        Exception.__init__(self, args, kwargs)


class PlayList(Service):
    """Class PlayList
    Introduction
    ============
    Implements a playlist for ZukeBox server as a service.
    For now the playlist works with a range of 10 positions, but in the 
    future that should change to range of undefined positions, thus
    the playlist will be "sizeable".
    """

    srvc_type = "urn:schemas-upnp-org:service:PlayList:1"
    srvc_name = "PlayList"

    def __init__(self, positions=10, xml_path=''):
        scpd_path = pjoin(xml_path, "zukebox-playlist-scpd.xml")
        log.info("PlayList service scpd_path = %s" % scpd_path)
        Service.__init__(self, self.srvc_name, self.srvc_type, '', scpd_path)

        self.positions = positions
        self.list = []
        self.current = 0
        self.prev = self.current
        self.next = None
        self.from_name = None
        self.to_name = None
        self.current_uri = None
        self.current_uri_metadata = None
        self.locked = 0
        self.available = 1

    def soap_IsLocked(self, *args, **kwargs):
        if len(self.list) == self.positions:
           self.locked = 1
        return {"Locked": self.locked}

    def soap_IsAvailable(self, *args, **kwargs):
        if not len(self.list) < 10:
            self.available = 0
        return {"Available": self.available}

    def soap_Append(self, *args, **kwargs):
        """Put a object in the playlist
        """
        if not self.locked:
            self.current_uri = kwargs["CurrentURI"]
            self.current_uri_metadata = kwargs["CurrentURIMetaData"]
            self.from_name = kwargs["FromName"]
            self.to_name = kwargs["ToName"]
            req = {"cur_uri": self.current_uri,
                    "cur_uri_data": self.current_uri_metadata,
                    "from_name": self.from_name,
                    "to_name": self.to_name}
            self.list.append(req)

            return {}
        else:
            raise PlayListOutBoundExcept("Playlist is full")

    def soap_Drop(self, *args, **kwargs):
        """Pop the object at position passed by index
        """
        if self.available:
            index = kwargs["Index"]
            self.list.pop(index)
            return {}
        else:
            raise PlayListOutBoundExcept("Playlist unavailable")

    def soap_GetSizeOfPlayList(self, *args, **kwargs):
        """Return the size of playlist"""
        lenght = len(self.list)
        return {"PlayListSize": lenght}

    def soap_GetCurrentItem(self, *args, **kwargs):
        """Get the current position from playlist"""
        if self.available:
            curr = self.list[self.current]
            #for now get only uri
            curr_uri = curr["cur_uri"]
            return {"CurrentItem": curr_uri}

    def soap_GetNextItem(self, *args, **kwargs):
        """Get the next item position from playlist"""
        if self.current == len(self.list) - 1:
            self.next = self.current
        else:
            self.next = self.current + 1
        self.current = self.next
        next = self.list[self.next]
        #for now get only next uri
        next_uri = next["cur_uri"]
        return {"NextItem": next_uri}

    def soap_GetPreviousItem(self, *args, **kwargs):
        """Get the previous item position from playlist"""
        if self.current != 0:
            self.prev = self.current - 1
            self.current = self.prev
        prev = self.list[self.prev]
        #for now get only previous uri
        prev_uri = prev["cur_uri"]
        return {"PreviousItem": prev_uri}

    def soap_GetPlaylist(self, *args, **kwargs):
        """Get the playlist"""
        return {"Playlist": self.list}


    def clean_playlist(self):
        if self.available:
            self.list = []

    def get_playlist(self):
        return self.list
